/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

### File Specific:
   main-programmer: christoph
   co-programmer:
*/


/* this is for debug output. It just says, that all calls to PRINT() belong to the DEBUG_MODULE_NETWORK module
   For more information refere to https://www.orxonox.net/cgi-bin/trac.cgi/wiki/DebugOutput
*/
#define DEBUG_MODULE_NETWORK

#include "handshake.h"

Handshake::Handshake( bool server, int clientId )
{
  /* set the class id for the base object */
  this->setClassID(CL_HANDSHAKE, "Handshake");

  this->server = server;
  this->clientId = clientId;
  this->state = HS_SEND_INIT;
  this->isOk = false;
}

void Handshake::writeBytes( const byte * data, int length )
{
  if ( state == HS_RECV_INIT )
  {
    if ( length != _INITIAL_DATA_LENGTH )
    {
      PRINTF(1)("initial packet has wrong size %d instead of %d\n", length, _INITIAL_DATA_LENGTH);
      state = HS_COMPLETED;
      return;
    }

    if ( strncmp((char*)data, _INITIAL_DATA, length) )
    {
      PRINTF(1)("initial packed does not match\n");
      state = HS_COMPLETED;
      return;
    }

    state = HS_SEND_VERSION;
    PRINTF(0)("got valid initial packet from client %d\n", clientId);
    return;
  }

  if ( state == HS_RECV_VERSION )
  {
    if ( length != _ORXONOX_VERSION_LENGTH )
    {
      PRINTF(1)("version number packet has wrong size %d instead of %d\n", length, _ORXONOX_VERSION_LENGTH);
      state = HS_COMPLETED;
      return;
    }

    if ( strncmp((char*)data, _ORXONOX_VERSION, length) )
    {
      PRINTF(1)("versions do not match\n");
      state = HS_COMPLETED;
      return;
    }

    if ( this->isServer() )
      state = HS_SEND_ID;
    else
      state = HS_RECV_ID;

    PRINTF(0)("client %d's version does match\n", clientId);
    return;
  }

  if ( state == HS_RECV_ID )
  {
    if ( length != _ORXONOX_VERSION_LENGTH )
    {
      PRINTF(1)("hostID packet has wrong size %d instead of %d\n", length, 4);
      state = HS_COMPLETED;
      return;
    }

    this->state = HS_COMPLETED;
    this->isOk = true;
    memcpy(&(this->newHostId), data, 4);
    PRINTF(0)("got my hostID: %d\n", newHostId);
    return;
  }

}

int Handshake::readBytes( byte * data, int maxLength, int & reciever )
{
  if ( state == HS_SEND_INIT )
  {
    if ( maxLength < _INITIAL_DATA_LENGTH )
    {
      PRINTF(1)("buffer too small for _INITIAL_DATA");
      state = HS_COMPLETED;
      return 0;
    }

    state = HS_RECV_INIT;
    memcpy(data, _INITIAL_DATA, _INITIAL_DATA_LENGTH);
    if ( this->isServer() )
      reciever = clientId;
    return _INITIAL_DATA_LENGTH;
  }

  if ( state == HS_SEND_VERSION )
  {
    if ( maxLength < _ORXONOX_VERSION_LENGTH )
    {
      PRINTF(1)("buffer too small for version number");
      state = HS_COMPLETED;
      return 0;
    }

    state = HS_RECV_VERSION;
    memcpy(data, _ORXONOX_VERSION, _ORXONOX_VERSION_LENGTH);
    if ( this->isServer() )
      reciever = clientId;
    return _ORXONOX_VERSION_LENGTH;
  }

  if ( state == HS_SEND_ID )
  {
    if ( maxLength < 4 )
    {
      PRINTF(1)("buffer too small for ID");
      state = HS_COMPLETED;
      return 0;
    }

    state = HS_COMPLETED;
    memcpy(data, (byte*)&clientId, 4);
    if ( this->isServer() )
      reciever = clientId;
    return 4;
  }

  return 0;
}

void Handshake::writeDebug( ) const
{
}

void Handshake::readDebug( ) const
{
}
