/*!
 * @file network_stream.h
 *  implementation of a network pipe
 */

#ifndef _HANDSHAKE
#define _HANDSHAKE

#include "base_object.h"
#include "synchronizeable.h"

#define _INITIAL_DATA "orxonox"
#define _INITIAL_DATA_LENGTH 7

#define _ORXONOX_VERSION "\x00\x00\x00\x01"
#define _ORXONOX_VERSION_LENGTH 4

typedef enum HandshakeState {
  HS_SEND_INIT = 0,
  HS_RECV_INIT,
  HS_SEND_VERSION,
  HS_RECV_VERSION,
  HS_SEND_ID,
  HS_RECV_ID,
  HS_COMPLETED,

  NUM_STATES
};

class Handshake : public Synchronizeable
{
  public:
    Handshake(bool server, int clientId = 0);
    inline bool       completed(){ return state == HS_COMPLETED; }
    inline bool       ok(){ return isOk; }
    inline int        getHostId(){ return newHostId; }

    virtual void      writeBytes(const byte* data, int length);
    virtual int       readBytes(byte* data, int maxLength, int& reciever);
    virtual void      writeDebug() const;
    virtual void      readDebug() const;

  private:
    int               state;
    int               server;
    int               clientId;
    int               newHostId;
    bool              isOk;

};


#endif
